structure Top =
struct
  fun parse file =
  let val ins = TextIO.openIn file in
    Parser.parse (Stream.fromTextInstream ins)
    before
    TextIO.closeIn ins
  end

  fun compile file =
  let val program = parse file in
    print "Finished parsing source code.\n";
    Compile.compile program
    before
    print "Finished compiling program.\n"
  end

  val compile_and_print = print o Assembly.pp_program o compile

  fun compile_and_run file args =
    let
      val program' = compile file
      val () = print "Running compiled program...\n"
      val (steps, result) = Assembly.run program' args
    in
      print ("Took " ^ Int.toString steps ^ " steps to run.\n");
      print (case result of
        Assembly.UnassignedVariable =>
          "Program execution ended early due to use of unassigned variable.\n"
      | Assembly.UnresolvedJump =>
          "Program execution ended early due to jump to nonexistent label.\n"
      | Assembly.NoReturn =>
          "Program execution ended early due to lack of return instruction.\n"
      | Assembly.DivideByZero =>
          "Program resulted in division by zero error.\n"
      | Assembly.Value n =>
          "Program returned " ^ Int.toString n ^ ".\n"
      )
    end

end
